@@ -17,6 +17,9 @@ from string import Template
 
 def main():
 
+  DEFAULT_CACHE_DIR = os.path.expanduser("~/.folderify/cache")
+  LOCAL_OSX_VERSION = ".".join(platform.mac_ver()[0].split(".")[:2])
+
   parser = argparse.ArgumentParser(
     description="Generate a native OSX folder icon from a mask file.",
     formatter_class=argparse.RawTextHelpFormatter
@@ -30,10 +33,10 @@ def main():
     nargs="?",
     type=str,
     help="Mask image file. For best results:\n\
-  - Use a .png mask.\n\
-  - Use a solid black design over a transparent background.\n\
-  - Make sure the corner pixels of the mask image are transparent. They are used for empty margins.\n\
-  - Make sure the icon is at least around 1024x1024, in order to look good at maximum Retina resolution.")
+- Use a .png mask.\n\
+- Use a solid black design over a transparent background.\n\
+- Make sure the corner pixels of the mask image are transparent. They are used for empty margins.\n\
+- Make sure the icon is at least around 1024x1024, in order to look good at maximum Retina resolution.")
 
   parser.add_argument(
     "target",
@@ -41,23 +44,22 @@ def main():
     nargs="?",
     type=str,
     help="Target file or folder. \
-  If a target is specified, the resulting icon will be applied to the target file/folder. \
-  Else, a .iconset folder and .icns file will be created in the same folder as mask \
-  (you can use \"Get Info\" in Finder to copy the icon from the .icns file).")
+If a target is specified, the resulting icon will be applied to the target file/folder. \
+Else, a .iconset folder and .icns file will be created in the same folder as mask \
+(you can use \"Get Info\" in Finder to copy the icon from the .icns file).")
 
   parser.add_argument(
     "--reveal", "-r",
     action="store_true",
     help="Reveal the target (or resulting .icns file) in Finder.")
 
-  local_osx_version = ".".join(platform.mac_ver()[0].split(".")[:2])
-
   parser.add_argument(
-    "--osx_version", "-x",
+    "--osx", "-x",
     type=str,
-    default=local_osx_version,
+    metavar="VERSION",
+    default=LOCAL_OSX_VERSION,
     help=("Version of the OSX folder icon, e.g. \"10.9\" or \"10.10\". \
-  Defaults to the version this computer is running (%s)." % local_osx_version))
+Defaults to the version this computer is running (%s)." % LOCAL_OSX_VERSION))
 
   parser.add_argument(
     "--cache", "-c",
@@ -67,21 +69,33 @@ def main():
   parser.add_argument(
     "--cache-dir",
     type=str,
-    default=os.path.expanduser("~/.folderify/cache"),
-    help="Cache directory.")
+    metavar="DIR",
+    default=DEFAULT_CACHE_DIR,
+    help="Use the specified cache directory (default: %s)." % DEFAULT_CACHE_DIR)
 
   exclusive.add_argument(
-    "--restore-from-cache",
+    "--cache-list",
+    action="store_true",
+    help="List all paths with cached masks.")
+
+  exclusive.add_argument(
+    "--cache-restore",
     metavar="PATH",
     type=str,
     help="Restore folderified icon to the file/folder at PATH,\
-  using the mask image in the cache for that path.")
+using the mask image in the cache for that path.")
 
   exclusive.add_argument(
-    "--restore-all-from-cache",
+    "--cache-restore-all",
     action="store_true",
     help="Restore all paths that have been cached.")
 
+  exclusive.add_argument(
+    "--cache-remove",
+    metavar="PATH",
+    type=str,
+    help="Remove the cached mask for the file/folder at PATH.")
+
 
   ################################################################
 
@@ -96,9 +110,9 @@ def main():
 
   data_folder = os.path.dirname(sys.modules[__name__].__file__)
 
-  if args.osx_version == "10.10":
+  if args.osx == "10.10":
     folder_type = "Yosemite"
-  elif args.osx_version in ["10.5", "10.6", "10.7", "10.8", "10.9"]:
+  elif args.osx in ["10.5", "10.6", "10.7", "10.8", "10.9"]:
     # http://arstechnica.com/apple/2007/10/mac-os-x-10-5/4/
     folder_type = "pre-Yosemite"
   else:
@@ -188,7 +202,6 @@ def main():
 
 
   def cache_path_for_target(target):
-    print args.cache_dir, target, ".mask"
     return args.cache_dir + os.path.abspath(target) + ".mask"
 
   def process_mask(mask, target=None, add_to_cache=False):
@@ -290,6 +303,13 @@ def main():
       add_to_cache=False
     )
 
+  def cached_targets():
+    for folder, _, files in os.walk(args.cache_dir):
+      for f in files:
+        if f.endswith(cached_mask_suffix):
+          cache_path = os.path.join(folder, f)
+          yield target_for_cache_path(cache_path)
+
   if args.mask:
     if args.cache:
       assert(args.target)
@@ -298,18 +318,24 @@ def main():
       target=args.target,
       add_to_cache=args.cache
     )
-  elif (args.restore_from_cache):
-    restore_from_cache(args.restore_from_cache)
-  elif (args.restore_all_from_cache):
-    for folder, _, files in os.walk(args.cache_dir):
-      for f in files:
-        if f.endswith(cached_mask_suffix):
-          cache_path = os.path.join(folder, f)
-          target = target_for_cache_path(cache_path)
-          if os.path.exists(target):
-            restore_from_cache(target)
-          else:
-            print "Target no longer exists: %s" % target
+
+  elif args.cache_restore:
+    restore_from_cache(args.cache_restore)
+
+  elif args.cache_remove:
+    os.remove(cache_path_for_target(args.cache_remove))
+
+  elif args.cache_restore_all:
+    for target in cached_targets():
+      if os.path.exists(target):
+        restore_from_cache(target)
+      else:
+        print "Target no longer exists: %s" % target
+
+  elif args.cache_list:
+    for target in cached_targets():
+      print target
+
   else:
     parser.print_help()
 
